@@ -1,7 +1,7 @@
 /*
- * @(#)WindowPeer.java	1.13 03/01/23
+ * @(#)WindowPeer.java	1.18 03/12/19
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 
@@ -20,5 +20,7 @@ import java.awt.event.KeyEvent;
 public interface WindowPeer extends ContainerPeer {
     void toFront();
     void toBack();
+    void updateAlwaysOnTop();
+    boolean requestWindowFocus();
 }
 
