@@ -1,7 +1,7 @@
 /*
- * @(#)Long.java	1.66 03/01/23
+ * @(#)Long.java	1.79 04/05/11
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 
@@ -19,12 +19,19 @@ package java.lang;
  * <code>String</code> to a <code>long</code>, as well as other
  * constants and methods useful when dealing with a <code>long</code>.
  *
+ * <p>Implementation note: The implementations of the "bit twiddling"
+ * methods (such as {@link #highestOneBit(long) highestOneBit} and
+ * {@link #numberOfTrailingZeros(long) numberOfTrailingZeros}) are
+ * based on material from Henry S. Warren, Jr.'s <i>Hacker's
+ * Delight</i>, (Addison Wesley, 2002).
+ *
  * @author  Lee Boynton
  * @author  Arthur van Hoff
- * @version 1.66, 01/23/03
+ * @author  Josh Bloch
+ * @version 1.79, 05/11/04
  * @since   JDK1.0
  */
-public final class Long extends Number implements Comparable {
+public final class Long extends Number implements Comparable<Long> {
     /**
      * A constant holding the minimum value a <code>long</code> can
      * have, -2<sup>63</sup>.
@@ -43,7 +50,7 @@ public final class Long extends Number implements Comparable {
      *
      * @since   JDK1.1
      */
-    public static final Class	TYPE = Class.getPrimitiveClass("long");
+    public static final Class<Long>	TYPE = (Class<Long>) Class.getPrimitiveClass("long");
 
     /**
      * Returns a string representation of the first argument in the
@@ -232,22 +239,25 @@ public final class Long extends Number implements Comparable {
     public static String toString(long i) {
         if (i == Long.MIN_VALUE)
             return "-9223372036854775808";
-        char[] buf = (char[])(perThreadBuffer.get());
-        int charPos = getChars(i, buf);
-        return new String(buf, charPos, (20 - charPos));
+        int size = (i < 0) ? stringSize(-i) + 1 : stringSize(i);
+        char[] buf = new char[size];
+        getChars(i, size, buf);
+        return new String(0, size, buf);
     }
 
-    // Per-thread buffer for string/stringbuffer conversion
-    private static ThreadLocal perThreadBuffer = new ThreadLocal() {
-        protected synchronized Object initialValue() {
-            return new char[20];
-        }
-    };
-
-    private static int getChars(long i, char[] buf) {
+    /**
+     * Places characters representing the integer i into the
+     * character array buf. The characters are placed into
+     * the buffer backwards starting with the least significant
+     * digit at the specified index (exclusive), and working
+     * backwards from there.
+     *
+     * Will fail if i == Long.MIN_VALUE
+     */
+    static void getChars(long i, int index, char[] buf) {
         long q;
         int r;
-        int charPos = 20;
+        int charPos = index;
         char sign = 0;
 
         if (i < 0) {
@@ -289,18 +299,17 @@ public final class Long extends Number implements Comparable {
         if (sign != 0) {
             buf[--charPos] = sign;
         }
-        return charPos;
     }
 
-    static void appendTo(long i, StringBuffer sb) {
-        if (i == Long.MIN_VALUE) {
-            sb.append("-9223372036854775808");
-            return;
+    // Requires positive x
+    static int stringSize(long x) {
+        long p = 10;
+        for (int i=1; i<19; i++) {
+            if (x < p)
+                return i;
+            p = 10*p;
         }
-        char[] buf = (char[])(perThreadBuffer.get());
-        int charPos = getChars(i, buf);
-        sb.append(buf, charPos, (20 - charPos));
-        return;
+        return 19;
     }
 
     /**
@@ -509,6 +518,38 @@ public final class Long extends Number implements Comparable {
 	return new Long(parseLong(s, 10));
     }
 
+    private static class LongCache {
+	private LongCache(){}
+
+	static final Long cache[] = new Long[-(-128) + 127 + 1];
+
+	static {
+	    for(int i = 0; i < cache.length; i++)
+		cache[i] = new Long(i - 128);
+	}
+    }
+
+    /**
+     * Returns a <tt>Long</tt> instance representing the specified
+     * <tt>long</tt> value.
+     * If a new <tt>Long</tt> instance is not required, this method
+     * should generally be used in preference to the constructor
+     * {@link #Long(long)}, as this method is likely to yield
+     * significantly better space and time performance by caching
+     * frequently requested values.
+     *
+     * @param  l a long value.
+     * @return a <tt>Long</tt> instance representing <tt>l</tt>.
+     * @since  1.5
+     */
+    public static Long valueOf(long l) {
+	final int offset = 128;
+	if (l >= -128 && l <= 127) { // will cache
+	    return LongCache.cache[(int)l + offset];
+	}
+        return new Long(l);
+    }
+
     /**
      * Decodes a <code>String</code> into a <code>Long</code>.
      * Accepts decimal, hexadecimal, and octal numbers given by the
@@ -600,7 +641,7 @@ public final class Long extends Number implements Comparable {
      *
      * @serial
      */
-    private long value;
+    private final long value;
 
     /**
      * Constructs a newly allocated <code>Long</code> object that
@@ -876,29 +917,227 @@ public final class Long extends Number implements Comparable {
 	return (thisVal<anotherVal ? -1 : (thisVal==anotherVal ? 0 : 1));
     }
 
+
+    // Bit Twiddling
+
     /**
-     * Compares this <code>Long</code> object to another object.  If
-     * the object is a <code>Long</code>, this function behaves like
-     * <code>compareTo(Long)</code>.  Otherwise, it throws a
-     * <code>ClassCastException</code> (as <code>Long</code> objects
-     * are comparable only to other <code>Long</code> objects).
+     * The number of bits used to represent a <tt>long</tt> value in two's
+     * complement binary form.
      *
-     * @param   o the <code>Object</code> to be compared.
-     * @return  the value <code>0</code> if the argument is a 
-     *		<code>Long</code> numerically equal to this 
-     *		<code>Long</code>; a value less than <code>0</code> 
-     *		if the argument is a <code>Long</code> numerically 
-     *		greater than this <code>Long</code>; and a value 
-     *		greater than <code>0</code> if the argument is a 
-     *		<code>Long</code> numerically less than this 
-     *		<code>Long</code>.
-     * @exception <code>ClassCastException</code> if the argument is not a
-     *		  <code>Long</code>.
-     * @see     java.lang.Comparable
-     * @since   1.2
+     * @since 1.5
      */
-    public int compareTo(Object o) {
-	return compareTo((Long)o);
+    public static final int SIZE = 64;
+ 
+    /**
+     * Returns an <tt>long</tt> value with at most a single one-bit, in the
+     * position of the highest-order ("leftmost") one-bit in the specified
+     * <tt>long</tt> value.  Returns zero if the specified value has no
+     * one-bits in its two's complement binary representation, that is, if it
+     * is equal to zero.
+     *
+     * @return an <tt>long</tt> value with a single one-bit, in the position
+     *     of the highest-order one-bit in the specified value, or zero if
+     *     the specified value is itself equal to zero.
+     * @since 1.5
+     */
+    public static long highestOneBit(long i) {
+        // HD, Figure 3-1
+        i |= (i >>  1);
+        i |= (i >>  2);
+        i |= (i >>  4);
+        i |= (i >>  8);
+        i |= (i >> 16);
+        i |= (i >> 32);
+        return i - (i >>> 1);
+    }
+
+    /**
+     * Returns an <tt>long</tt> value with at most a single one-bit, in the
+     * position of the lowest-order ("rightmost") one-bit in the specified
+     * <tt>long</tt> value.  Returns zero if the specified value has no
+     * one-bits in its two's complement binary representation, that is, if it
+     * is equal to zero.
+     *
+     * @return an <tt>long</tt> value with a single one-bit, in the position
+     *     of the lowest-order one-bit in the specified value, or zero if
+     *     the specified value is itself equal to zero.
+     * @since 1.5
+     */
+    public static long lowestOneBit(long i) {
+        // HD, Section 2-1
+        return i & -i;
+    }
+ 
+    /**
+     * Returns the number of zero bits preceding the highest-order
+     * ("leftmost") one-bit in the two's complement binary representation
+     * of the specified <tt>long</tt> value.  Returns 64 if the
+     * specified value has no one-bits in its two's complement representation,
+     * in other words if it is equal to zero.
+     *
+     * <p>Note that this method is closely related to the logarithm base 2.
+     * For all positive <tt>long</tt> values x:
+     * <ul>
+     * <li>floor(log<sub>2</sub>(x)) = <tt>63 - numberOfLeadingZeros(x)</tt>
+     * <li>ceil(log<sub>2</sub>(x)) = <tt>64 - numberOfLeadingZeros(x - 1)</tt>
+     * </ul>
+     *
+     * @return the number of zero bits preceding the highest-order
+     *     ("leftmost") one-bit in the two's complement binary representation
+     *     of the specified <tt>long</tt> value, or 64 if the value
+     *     is equal to zero.
+     * @since 1.5
+     */
+    public static int numberOfLeadingZeros(long i) {
+        // HD, Figure 5-6
+         if (i == 0)
+            return 64;
+        int n = 1;
+	int x = (int)(i >>> 32);
+        if (x == 0) { n += 32; x = (int)i; }
+        if (x >>> 16 == 0) { n += 16; x <<= 16; }
+        if (x >>> 24 == 0) { n +=  8; x <<=  8; }
+        if (x >>> 28 == 0) { n +=  4; x <<=  4; }
+        if (x >>> 30 == 0) { n +=  2; x <<=  2; }
+        n -= x >>> 31;
+        return n;
+    }
+ 
+    /**
+     * Returns the number of zero bits following the lowest-order ("rightmost")
+     * one-bit in the two's complement binary representation of the specified
+     * <tt>long</tt> value.  Returns 64 if the specified value has no
+     * one-bits in its two's complement representation, in other words if it is
+     * equal to zero.
+     *
+     * @return the number of zero bits following the lowest-order ("rightmost")
+     *     one-bit in the two's complement binary representation of the
+     *     specified <tt>long</tt> value, or 64 if the value is equal
+     *     to zero.
+     * @since 1.5
+     */
+    public static int numberOfTrailingZeros(long i) {
+        // HD, Figure 5-14
+	int x, y;
+	if (i == 0) return 64;
+	int n = 63;
+	y = (int)i; if (y != 0) { n = n -32; x = y; } else x = (int)(i>>>32);
+	y = x <<16; if (y != 0) { n = n -16; x = y; }
+	y = x << 8; if (y != 0) { n = n - 8; x = y; }
+	y = x << 4; if (y != 0) { n = n - 4; x = y; }
+	y = x << 2; if (y != 0) { n = n - 2; x = y; }
+	return n - ((x << 1) >>> 31);
+    }
+
+    /**
+     * Returns the number of one-bits in the two's complement binary
+     * representation of the specified <tt>long</tt> value.  This function is
+     * sometimes referred to as the <i>population count</i>.
+     *
+     * @return the number of one-bits in the two's complement binary
+     *     representation of the specified <tt>long</tt> value.
+     * @since 1.5
+     */
+     public static int bitCount(long i) {
+        // HD, Figure 5-14
+	i = i - ((i >>> 1) & 0x5555555555555555L);
+	i = (i & 0x3333333333333333L) + ((i >>> 2) & 0x3333333333333333L);
+	i = (i + (i >>> 4)) & 0x0f0f0f0f0f0f0f0fL;
+	i = i + (i >>> 8);
+	i = i + (i >>> 16);
+	i = i + (i >>> 32);
+	return (int)i & 0x7f;
+     }
+ 
+    /**
+     * Returns the value obtained by rotating the two's complement binary
+     * representation of the specified <tt>long</tt> value left by the
+     * specified number of bits.  (Bits shifted out of the left hand, or
+     * high-order, side reenter on the right, or low-order.)
+     *
+     * <p>Note that left rotation with a negative distance is equivalent to
+     * right rotation: <tt>rotateLeft(val, -distance) == rotateRight(val,
+     * distance)</tt>.  Note also that rotation by any multiple of 64 is a
+     * no-op, so all but the last six bits of the rotation distance can be
+     * ignored, even if the distance is negative: <tt>rotateLeft(val,
+     * distance) == rotateLeft(val, distance & 0x3F)</tt>.
+     *
+     * @return the value obtained by rotating the two's complement binary
+     *     representation of the specified <tt>long</tt> value left by the
+     *     specified number of bits.
+     * @since 1.5
+     */
+    public static long rotateLeft(long i, int distance) {
+        return (i << distance) | (i >>> -distance);
+    }
+ 
+    /**
+     * Returns the value obtained by rotating the two's complement binary
+     * representation of the specified <tt>long</tt> value right by the
+     * specified number of bits.  (Bits shifted out of the right hand, or
+     * low-order, side reenter on the left, or high-order.)
+     *
+     * <p>Note that right rotation with a negative distance is equivalent to
+     * left rotation: <tt>rotateRight(val, -distance) == rotateLeft(val,
+     * distance)</tt>.  Note also that rotation by any multiple of 64 is a
+     * no-op, so all but the last six bits of the rotation distance can be
+     * ignored, even if the distance is negative: <tt>rotateRight(val,
+     * distance) == rotateRight(val, distance & 0x3F)</tt>.
+     *
+     * @return the value obtained by rotating the two's complement binary
+     *     representation of the specified <tt>long</tt> value right by the
+     *     specified number of bits.
+     * @since 1.5
+     */
+    public static long rotateRight(long i, int distance) {
+        return (i >>> distance) | (i << -distance);
+    }
+ 
+    /**
+     * Returns the value obtained by reversing the order of the bits in the
+     * two's complement binary representation of the specified <tt>long</tt>
+     * value.
+     *
+     * @return the value obtained by reversing order of the bits in the
+     *     specified <tt>long</tt> value.
+     * @since 1.5
+     */
+    public static long reverse(long i) {
+        // HD, Figure 7-1
+	i = (i & 0x5555555555555555L) << 1 | (i >>> 1) & 0x5555555555555555L;
+	i = (i & 0x3333333333333333L) << 2 | (i >>> 2) & 0x3333333333333333L;
+	i = (i & 0x0f0f0f0f0f0f0f0fL) << 4 | (i >>> 4) & 0x0f0f0f0f0f0f0f0fL;
+	i = (i & 0x00ff00ff00ff00ffL) << 8 | (i >>> 8) & 0x00ff00ff00ff00ffL;
+	i = (i << 48) | ((i & 0xffff0000L) << 16) |
+	    ((i >>> 16) & 0xffff0000L) | (i >>> 48);
+	return i;
+    }
+ 
+    /**
+     * Returns the signum function of the specified <tt>long</tt> value.  (The
+     * return value is -1 if the specified value is negative; 0 if the
+     * specified value is zero; and 1 if the specified value is positive.)
+     *
+     * @return the signum function of the specified <tt>long</tt> value.
+     * @since 1.5
+     */
+    public static int signum(long i) {
+        // HD, Section 2-7
+        return (int) ((i >> 63) | (-i >>> 63));
+    }
+ 
+    /**
+     * Returns the value obtained by reversing the order of the bytes in the
+     * two's complement representation of the specified <tt>long</tt> value.
+     *
+     * @return the value obtained by reversing the bytes in the specified
+     *     <tt>long</tt> value.
+     * @since 1.5
+     */
+    public static long reverseBytes(long i) {
+        i = (i & 0x00ff00ff00ff00ffL) << 8 | (i >>> 8) & 0x00ff00ff00ff00ffL;
+        return (i << 48) | ((i & 0xffff0000L) << 16) |
+            ((i >>> 16) & 0xffff0000L) | (i >>> 48);
     }
 
     /** use serialVersionUID from JDK 1.0.2 for interoperability */
