@@ -1,7 +1,7 @@
 /*
- * @(#)SocketPermission.java	1.56 03/04/16
+ * @(#)SocketPermission.java	1.59 04/02/12
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
  
@@ -21,6 +21,7 @@ import java.io.ObjectStreamField;
 import java.io.ObjectOutputStream;
 import java.io.ObjectInputStream;
 import java.io.IOException;
+import sun.net.util.IPAddressUtil;
 import sun.security.util.SecurityConstants;
 
 
@@ -109,7 +110,7 @@ import sun.security.util.SecurityConstants;
  * @see java.security.Permissions
  * @see SocketPermission
  *
- * @version 1.56 03/04/16
+ * @version 1.59 04/02/12
  *
  * @author Marianne Mueller
  * @author Roland Schemers 
@@ -398,9 +399,9 @@ implements java.io.Serializable
 	        // see if we are being initialized with an IP address.
 	        char ch = host.charAt(0);
 	        if (ch == ':' || Character.digit(ch, 16) != -1) {
-		    byte ip[] = Inet4Address.textToNumericFormat(host);
+		    byte ip[] = IPAddressUtil.textToNumericFormatV4(host);
 		    if (ip == null) {
-		        ip = Inet6Address.textToNumericFormat(host);
+		        ip = IPAddressUtil.textToNumericFormatV6(host);
 		    }
 		    if (ip != null) {
 		        try {
@@ -911,7 +912,7 @@ implements java.io.Serializable
      */
     private static String getActions(int mask)
     {
-	StringBuffer sb = new StringBuffer();
+	StringBuilder sb = new StringBuilder();
         boolean comma = false;
 
 	if ((mask & CONNECT) == CONNECT) {
@@ -1049,7 +1050,7 @@ else its the cname?
  * @see java.security.Permissions
  * @see java.security.PermissionCollection
  *
- * @version 1.56 04/16/03
+ * @version 1.59 02/12/04
  *
  * @author Roland Schemers
  *
@@ -1090,14 +1091,14 @@ implements Serializable
 	    throw new IllegalArgumentException("invalid permission: "+
 					       permission);
 	if (isReadOnly())
-	    throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
-
-        // No need to synchronize because all adds are done sequentially
-	// before any implies() calls
+	    throw new SecurityException(
+		"attempt to add a Permission to a readonly PermissionCollection");
 
 	// optimization to ensure perms most likely to be tested
 	// show up early (4301064)
-	perms.add(0, permission);
+	synchronized (this) {
+	    perms.add(0, permission);
+	}
     }
 
     /**
@@ -1121,16 +1122,18 @@ implements Serializable
 	int effective = 0;
 	int needed = desired;
 
-	int len = perms.size();
-	//System.out.println("implies "+np);
-	for (int i = 0; i < len; i++) {
-	    SocketPermission x = (SocketPermission) perms.get(i);
-	    //System.out.println("  trying "+x);
-	    if (((needed & x.getMask()) != 0) && x.impliesIgnoreMask(np)) {
-		effective |=  x.getMask();
-		if ((effective & desired) == desired)
-		    return true;
-		needed = (desired ^ effective);
+	synchronized (this) {
+	    int len = perms.size();
+	    //System.out.println("implies "+np);
+	    for (int i = 0; i < len; i++) {
+		SocketPermission x = (SocketPermission) perms.get(i);
+		//System.out.println("  trying "+x);
+		if (((needed & x.getMask()) != 0) && x.impliesIgnoreMask(np)) {
+		    effective |=  x.getMask();
+		    if ((effective & desired) == desired)
+			return true;
+		    needed = (desired ^ effective);
+		}
 	    }
 	}
 	return false;
@@ -1145,7 +1148,9 @@ implements Serializable
 
     public Enumeration elements() {
         // Convert Iterator into Enumeration
-	return Collections.enumeration(perms);
+	synchronized (this) {
+	    return Collections.enumeration(perms);
+	}
     }
 
     private static final long serialVersionUID = 2787186408602843674L;
@@ -1179,7 +1184,10 @@ implements Serializable
 
 	// Write out Vector
 	Vector permissions = new Vector(perms.size());
-	permissions.addAll(perms);
+
+	synchronized (this) {
+	    permissions.addAll(perms);
+	}
 
         ObjectOutputStream.PutField pfields = out.putFields();
         pfields.put("permissions", permissions);
