@@ -1,7 +1,7 @@
 /*
- * @(#)MotifFileChooserUI.java	1.37 03/01/23
+ * @(#)MotifFileChooserUI.java	1.44 04/09/10
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 
@@ -22,7 +22,7 @@ import java.util.*;
 /**
  * Motif FileChooserUI.
  *
- * @version 1.37 01/23/03
+ * @version 1.44 09/10/04
  * @author Jeff Dinkins
  */
 public class MotifFileChooserUI extends BasicFileChooserUI {
@@ -55,6 +55,7 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
     private static final Insets buttonMargin = new Insets(3, 3, 3, 3);
 
     private JPanel directoryPanel = new JPanel();
+    private JPanel bottomPanel;
 
     protected JButton approveButton;
 
@@ -73,6 +74,36 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
     private String filterLabelText = null;
     private int filterLabelMnemonic = 0;
 
+    private String fileNameString(File file) {
+        if (file == null) {
+            return null;
+        } else {
+            JFileChooser fc = getFileChooser();
+            if (fc.isDirectorySelectionEnabled() && !fc.isFileSelectionEnabled()) {
+                return file.getPath();
+            } else {
+                return file.getName();
+            }
+        }
+    }
+
+    private String fileNameString(File[] files) {
+        StringBuffer buf = new StringBuffer();
+        for (int i = 0; files != null && i < files.length; i++) {
+            if (i > 0) {
+                buf.append(" ");
+            }
+            if (files.length > 1) {
+                buf.append("\"");
+            }
+            buf.append(fileNameString(files[i]));
+            if (files.length > 1) {
+                buf.append("\"");
+            }
+        }
+        return buf.toString();
+    }
+
     public MotifFileChooserUI(JFileChooser filechooser) {
 	super(filechooser);
     }
@@ -104,7 +135,7 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
     }
 
     public void rescanCurrentDirectory(JFileChooser fc) {
-	// PENDING(jeff)
+        getModel().validateFileCache();
     }
 
     public PropertyChangeListener createPropertyChangeListener(JFileChooser fc) {
@@ -116,9 +147,26 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 		    if(f != null) {
 			setFileName(getFileChooser().getName(f));
 		    }
+        } 
+        else if (prop.equals(JFileChooser.SELECTED_FILES_CHANGED_PROPERTY)) {
+            File[] files = (File[]) e.getNewValue();
+            JFileChooser fc = getFileChooser();
+            if (files != null && files.length > 0 && (files.length > 1 || fc.isDirectorySelectionEnabled() || !files[0].isDirectory())) {
+               setFileName(fileNameString(files)); 
+            }
 		} else if(prop.equals(JFileChooser.DIRECTORY_CHANGED_PROPERTY)) {
 		    directoryList.clearSelection();
+                    ListSelectionModel sm = directoryList.getSelectionModel();
+                    if (sm instanceof DefaultListSelectionModel) {
+                        ((DefaultListSelectionModel)sm).moveLeadSelectionIndex(0);       
+                        ((DefaultListSelectionModel)sm).setAnchorSelectionIndex(0);
+                    }
 		    fileList.clearSelection();
+                    sm = fileList.getSelectionModel();
+                    if (sm instanceof DefaultListSelectionModel) {
+                        ((DefaultListSelectionModel)sm).moveLeadSelectionIndex(0);
+                        ((DefaultListSelectionModel)sm).setAnchorSelectionIndex(0);
+                    }
 		    File currentDirectory = getFileChooser().getCurrentDirectory();
 		    if(currentDirectory != null) {
 			try {
@@ -157,7 +205,9 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 			   prop == JFileChooser.DIALOG_TYPE_CHANGED_PROPERTY) {
 		    approveButton.setText(getApproveButtonText(getFileChooser()));
 		    approveButton.setToolTipText(getApproveButtonToolTipText(getFileChooser()));
-		} else if (prop.equals("componentOrientation")) {
+		} else if (prop.equals(JFileChooser.CONTROL_BUTTONS_ARE_SHOWN_CHANGED_PROPERTY)) {
+                    doControlButtonsChanged(e);
+                } else if (prop.equals("componentOrientation")) {
 		    ComponentOrientation o = (ComponentOrientation)e.getNewValue();
 		    JFileChooser cc = (JFileChooser)e.getSource();
 		    if (o != (ComponentOrientation)e.getOldValue()) {
@@ -324,7 +374,7 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	filenameTextField.setAlignmentX(JComponent.LEFT_ALIGNMENT);
 	interior.add(filenameTextField);
 
-	JPanel bottomPanel = new JPanel(new BorderLayout(0, 4));
+	bottomPanel = getBottomPanel();
 	bottomPanel.add(new JSeparator(), BorderLayout.NORTH);
 
 	// Add buttons
@@ -386,7 +436,24 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	buttonPanel.add(Box.createGlue());
 
 	bottomPanel.add(buttonPanel, BorderLayout.SOUTH);
-	fc.add(bottomPanel, BorderLayout.SOUTH);
+	if (fc.getControlButtonsAreShown()) {
+           fc.add(bottomPanel, BorderLayout.SOUTH);
+        }
+    }
+
+    protected JPanel getBottomPanel() {
+        if (bottomPanel == null) {
+            bottomPanel = new JPanel(new BorderLayout(0, 4));
+        }
+        return bottomPanel;
+    }
+
+    private void doControlButtonsChanged(PropertyChangeEvent e) {
+        if (getFileChooser().getControlButtonsAreShown()) {
+            getFileChooser().add(bottomPanel,BorderLayout.SOUTH);
+        } else {
+            getFileChooser().remove(getBottomPanel());
+        }
     }
 
     public void uninstallComponents(JFileChooser fc) {
@@ -434,6 +501,7 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	}
 	
 	fileList.setModel(new MotifFileListModel());
+        fileList.getSelectionModel().removeSelectionInterval(0, 0);
 	fileList.setCellRenderer(new FileCellRenderer());
 	fileList.addListSelectionListener(createListSelectionListener(getFileChooser()));
 	fileList.addMouseListener(createDoubleClickListener(getFileChooser(), fileList));
@@ -451,6 +519,7 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 
 	directoryList.setCellRenderer(new DirectoryCellRenderer());
 	directoryList.setModel(new MotifDirectoryListModel());
+        directoryList.getSelectionModel().removeSelectionInterval(0, 0);
 	directoryList.addMouseListener(createDoubleClickListener(getFileChooser(), directoryList));
 	directoryList.addListSelectionListener(createListSelectionListener(getFileChooser()));
 
@@ -520,10 +589,11 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	}
 
 	public void intervalAdded(ListDataEvent e) {
+            fireIntervalAdded(this, e.getIndex0(), e.getIndex1());
 	}
 
-	// PENDING(jeff) - implement
 	public void intervalRemoved(ListDataEvent e) {
+            fireIntervalRemoved(this, e.getIndex0(), e.getIndex1());
 	}
 
 	// PENDING(jeff) - this is inefficient - should sent out
@@ -563,10 +633,11 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	}
 
 	public void intervalAdded(ListDataEvent e) {
+            fireIntervalAdded(this, e.getIndex0(), e.getIndex1());
 	}
 
-	// PENDING(jeff) - implement
 	public void intervalRemoved(ListDataEvent e) {
+            fireIntervalRemoved(this, e.getIndex0(), e.getIndex1());
 	}
 
 	// PENDING(jeff) - this is inefficient - should sent out
@@ -631,6 +702,8 @@ public class MotifFileChooserUI extends BasicFileChooserUI {
 	    if(prop == JFileChooser.CHOOSABLE_FILE_FILTER_CHANGED_PROPERTY) {
 		filters = (FileFilter[]) e.getNewValue();
 		fireContentsChanged(this, -1, -1);
+	    } else if (prop == JFileChooser.FILE_FILTER_CHANGED_PROPERTY) {
+		fireContentsChanged(this, -1, -1);
 	    }
 	}
 
