@@ -1,7 +1,7 @@
 /*
- * @(#)TTY.java	1.54 03/01/23
+ * @(#)TTY.java	1.61 04/04/12
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 /*
@@ -55,12 +55,6 @@ public class TTY implements EventNotifier {
      */
     private static final String progname = "jdb";
 
-    /**
-     * The version date of this tool.  Formats to:
-     *      April 19, 2002 7:42:13 PM PDT (when locale=en_US.ISO8859-1)
-     */
-    private static final long version = 1019270533724L;
-
     public void vmStartEvent(VMStartEvent se)  {
         Thread.yield();  // fetch output
         MessageOutput.lnprint("VM Started:");
@@ -195,24 +189,122 @@ public class TTY implements EventNotifier {
         MessageOutput.println("zz help text");
     }
 
-    /*
-     * These commands do not require a connected VM
-     */
-    private String[]  disconnectCmds = {
-        "run", "catch", "ignore", "stop", "clear", "watch", "unwatch",
-        "use", "sourcepath", "quit", "exit", "help", "?", "read",
-        "version", "exclude"
+    private static final String[][] commandList = {
+        /* 
+         * NOTE: this list must be kept sorted in ascending ASCII
+         *       order by element [0].  Ref: isCommand() below.
+         *
+         *Command      OK when        OK when
+         * name      disconnected?   readonly?
+         *------------------------------------
+         */
+        {"!!",           "n",         "y"},
+        {"?",            "y",         "y"},
+        {"bytecodes",    "n",         "y"},
+        {"catch",        "y",         "n"},
+        {"class",        "n",         "y"},
+        {"classes",      "n",         "y"},
+        {"classpath",    "n",         "y"},
+        {"clear",        "y",         "n"},
+        {"connectors",   "y",         "y"},
+        {"cont",         "n",         "n"},
+        {"disablegc",    "n",         "n"},
+        {"down",         "n",         "y"},
+        {"dump",         "n",         "y"},
+        {"enablegc",     "n",         "n"},
+        {"eval",         "n",         "y"},
+        {"exclude",      "y",         "n"},
+        {"exit",         "y",         "y"},
+        {"extension",    "n",         "y"},
+        {"fields",       "n",         "y"},
+        {"gc",           "n",         "n"},
+        {"help",         "y",         "y"},
+        {"ignore",       "y",         "n"},
+        {"interrupt",    "n",         "n"},
+        {"kill",         "n",         "n"},
+        {"lines",        "n",         "y"},
+        {"list",         "n",         "y"},
+        {"load",         "n",         "y"},
+        {"locals",       "n",         "y"},
+        {"lock",         "n",         "n"},
+        {"memory",       "n",         "y"},
+        {"methods",      "n",         "y"},
+        {"monitor",      "n",         "n"},
+        {"next",         "n",         "n"},
+        {"pop",          "n",         "n"},
+        {"print",        "n",         "y"},
+        {"quit",         "y",         "y"},
+        {"read",         "y",         "y"},
+        {"redefine",     "n",         "n"},
+        {"reenter",      "n",         "n"},
+        {"resume",       "n",         "n"},
+        {"run",          "y",         "n"},
+        {"save",         "n",         "n"},
+        {"set",          "n",         "n"},
+        {"sourcepath",   "y",         "y"},
+        {"step",         "n",         "n"},
+        {"stepi",        "n",         "n"},
+        {"stop",         "y",         "n"},
+        {"suspend",      "n",         "n"},
+        {"thread",       "n",         "y"},
+        {"threadgroup",  "n",         "y"},
+        {"threadgroups", "n",         "y"},
+        {"threadlocks",  "n",         "y"},
+        {"threads",      "n",         "y"},
+        {"trace",        "n",         "n"},
+        {"unmonitor",    "n",         "n"},
+        {"untrace",      "n",         "n"},
+        {"unwatch",      "y",         "n"},
+        {"up",           "n",         "y"},
+        {"use",          "y",         "y"},
+        {"version",      "y",         "y"},
+        {"watch",        "y",         "n"},
+        {"where",        "n",         "y"},
+        {"wherei",       "n",         "y"},
     };
 
-    private boolean isDisconnectCmd(String cmd) {
-        for (int i = 0; i < disconnectCmds.length; i++) {
-            if (disconnectCmds[i].equals(cmd)) {
-                return true;
-            }
-        }
-        return false;
+    /*
+     * Look up the command string in commandList.
+     * If found, return the index.
+     * If not found, return index < 0
+     */
+    private int isCommand(String key) {
+        //Reference: binarySearch() in java/util/Arrays.java
+        //           Adapted for use with String[][0].
+        int low = 0;
+	int high = commandList.length - 1;
+        long i = 0;
+	while (low <= high) {
+	    int mid = (low + high) >> 1;
+	    String midVal = commandList[mid][0];
+            int compare = midVal.compareTo(key);
+	    if (compare < 0)
+		low = mid + 1;
+	    else if (compare > 0)
+		high = mid - 1;
+	    else
+		return mid; // key found
+	}
+	return -(low + 1);  // key not found.
+    };
+
+    /*
+     * Return true if the command is OK when disconnected. 
+     */
+    private boolean isDisconnectCmd(int ii) {
+        if (ii < 0 || ii >= commandList.length) return false;
+        return (commandList[ii][1].equals("y"));
     }
 
+    /*
+     * Return true if the command is OK when readonly.
+     */
+    private boolean isReadOnlyCmd(int ii) {
+        if (ii < 0 || ii >= commandList.length) return false;
+        return (commandList[ii][2].equals("y"));
+    };
+
+
     void executeCommand(StringTokenizer t) {
         String cmd = t.nextToken().toLowerCase();
         Commands evaluator = new Commands();
@@ -221,173 +313,189 @@ public class TTY implements EventNotifier {
         boolean showPrompt = true;
 
         /*
-         * Do this check ahead of time so that it precedes any syntax
-         * checking in the command itself. If the command is not
-         * properly classified by isDisconnectCmd, the exception catch
-         * below will still protect us. 
+         * Check for an integer repetition prefix.  If found,
+         * recursively execute cmd that number of times.
          */
-        if (!Env.connection().isOpen() && !isDisconnectCmd(cmd)) {
-            MessageOutput.println("Command not valid until the VM is started with the run command",
-                         cmd);
-        } else {
+        if (Character.isDigit(cmd.charAt(0)) && t.hasMoreTokens()) {
             try {
-                if (cmd.equals("print")) {
-                    evaluator.commandPrint(t, false);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("eval")) {
-                    evaluator.commandPrint(t, false);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("set")) {
-                    evaluator.commandSet(t);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("dump")) {
-                    evaluator.commandPrint(t, true);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("locals")) {
-                    evaluator.commandLocals();
-                } else if (cmd.equals("classes")) {
-                    evaluator.commandClasses();
-                } else if (cmd.equals("class")) {
-                    evaluator.commandClass(t);
-                } else if (cmd.equals("methods")) {
-                    evaluator.commandMethods(t);
-                } else if (cmd.equals("fields")) {
-                    evaluator.commandFields(t);
-                } else if (cmd.equals("threads")) {
-                    evaluator.commandThreads(t);
-                } else if (cmd.equals("thread")) {
-                    evaluator.commandThread(t);
-                } else if (cmd.equals("suspend")) {
-                    evaluator.commandSuspend(t);
-                } else if (cmd.equals("resume")) {
-                    evaluator.commandResume(t);
-                } else if (cmd.equals("cont")) {
-                    evaluator.commandCont();
-                } else if (cmd.equals("threadgroups")) {
-                    evaluator.commandThreadGroups();
-                } else if (cmd.equals("threadgroup")) {
-                    evaluator.commandThreadGroup(t);
-                } else if (cmd.equals("catch")) {
-                    evaluator.commandCatchException(t);
-                } else if (cmd.equals("ignore")) {
-                    evaluator.commandIgnoreException(t);
-                } else if (cmd.equals("step")) {
-                    evaluator.commandStep(t);
-                } else if (cmd.equals("stepi")) {
-                    evaluator.commandStepi();
-                } else if (cmd.equals("next")) {
-                    evaluator.commandNext();
-                } else if (cmd.equals("kill")) {
-                    evaluator.commandKill(t);
-                } else if (cmd.equals("interrupt")) {
-                    evaluator.commandInterrupt(t);
-                } else if (cmd.equals("trace")) {
-                    evaluator.commandTrace(t);
-                } else if (cmd.equals("untrace")) {
-                    evaluator.commandUntrace(t);
-                } else if (cmd.equals("where")) {
-                    evaluator.commandWhere(t, false);
-                } else if (cmd.equals("wherei")) {
-                    evaluator.commandWhere(t, true);
-                } else if (cmd.equals("up")) {
-                    evaluator.commandUp(t);
-                } else if (cmd.equals("down")) {
-                    evaluator.commandDown(t);
-                } else if (cmd.equals("load")) {
-                    evaluator.commandLoad(t);
-                } else if (cmd.equals("run")) {
-                    evaluator.commandRun(t);
-                    /*
-                     * Fire up an event handler, if the connection was just
-                     * opened. Since this was done from the run command
-                     * we don't stop the VM on its VM start event (so
-                     * arg 2 is false).
-                     */
-                    if ((handler == null) && Env.connection().isOpen()) {
-                        handler = new EventHandler(this, false);
-                    }
-                } else if (cmd.equals("memory")) {
-                    evaluator.commandMemory();
-                } else if (cmd.equals("gc")) {
-                    evaluator.commandGC();
-                } else if (cmd.equals("stop")) {
-                    evaluator.commandStop(t);
-                } else if (cmd.equals("clear")) {
-                    evaluator.commandClear(t);
-                } else if (cmd.equals("watch")) {
-                    evaluator.commandWatch(t);
-                } else if (cmd.equals("unwatch")) {
-                    evaluator.commandUnwatch(t);
-                } else if (cmd.equals("list")) {
-                    evaluator.commandList(t);
-                } else if (cmd.equals("lines")) { // Undocumented command: useful for testing.
-                    evaluator.commandLines(t);
-                } else if (cmd.equals("classpath")) {
-                    evaluator.commandClasspath(t);
-                } else if (cmd.equals("use") || cmd.equals("sourcepath")) {
-                    evaluator.commandUse(t);
-                } else if (cmd.equals("monitor")) {
-                    monitorCommand(t);
-                } else if (cmd.equals("unmonitor")) {
-                    unmonitorCommand(t);
-                } else if (cmd.equals("lock")) {
-                    evaluator.commandLock(t);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("threadlocks")) {
-                    evaluator.commandThreadlocks(t);
-                } else if (cmd.equals("disablegc")) {
-                    evaluator.commandDisableGC(t);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("enablegc")) {
-                    evaluator.commandEnableGC(t);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("save")) { // Undocumented command: useful for testing.
-                    evaluator.commandSave(t);
-                    showPrompt = false;        // asynchronous command
-                } else if (cmd.equals("bytecodes")) { // Undocumented command: useful for testing.
-                    evaluator.commandBytecodes(t);
-                } else if (cmd.equals("redefine")) {
-                    evaluator.commandRedefine(t);
-                } else if (cmd.equals("pop")) {
-                    evaluator.commandPopFrames(t, false);
-                } else if (cmd.equals("reenter")) {
-                    evaluator.commandPopFrames(t, true);
-                } else if (cmd.equals("extension")) {
-                    evaluator.commandExtension(t);
-                } else if (cmd.equals("exclude")) {
-                    evaluator.commandExclude(t);
-                } else if (cmd.equals("read")) {
-                    readCommand(t);
-                } else if (cmd.equals("help") || cmd.equals("?")) {
-                    help();
-                } else if (cmd.equals("version")) {
-                    evaluator.commandVersion(progname, version);
-                } else if (cmd.equals("quit") || cmd.equals("exit")) {
-                    if (handler != null) {
-                        handler.shutdown();
-                    }
-                    Env.shutdown(); 
-                } else {
-                    if (t.hasMoreTokens()) {
-                        try {            
-                            int repeat = Integer.parseInt(cmd);
-                            String subcom = t.nextToken("");
-                            while (repeat-- > 0) {
-                                executeCommand(new StringTokenizer(subcom));
-                            }
-                            return;
-                        } catch (NumberFormatException exc) {
-                        }
-                    }
-                    MessageOutput.println("Unrecognized command.  Try help...");
+                int repeat = Integer.parseInt(cmd);
+                String subcom = t.nextToken("");
+                while (repeat-- > 0) {
+                    executeCommand(new StringTokenizer(subcom));
+                    showPrompt = false; // Bypass the printPrompt() below.
                 }
-            } catch (UnsupportedOperationException uoe) {
-                MessageOutput.println("Command is not supported on the target VM", cmd);
-            } catch (VMNotConnectedException vmnse) {
+            } catch (NumberFormatException exc) {
+                MessageOutput.println("Unrecognized command.  Try help...", cmd);
+            }
+        } else {
+            int commandNumber = isCommand(cmd);
+            /*
+             * Check for an unknown command
+             */
+            if (commandNumber < 0) {
+                MessageOutput.println("Unrecognized command.  Try help...", cmd);
+            } else if (!Env.connection().isOpen() && !isDisconnectCmd(commandNumber)) {
                 MessageOutput.println("Command not valid until the VM is started with the run command",
-                             cmd);
-            } catch (Exception e) {
-                MessageOutput.printException("Internal exception:", e);
+                                      cmd);
+            } else if (Env.connection().isOpen() && !Env.vm().canBeModified() &&
+                       !isReadOnlyCmd(commandNumber)) {
+                MessageOutput.println("Command is not supported on a read-only VM connection",
+                                      cmd);
+            } else {
+
+                try {
+                    if (cmd.equals("print")) {
+                        evaluator.commandPrint(t, false);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("eval")) {
+                        evaluator.commandPrint(t, false);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("set")) {
+                        evaluator.commandSet(t);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("dump")) {
+                        evaluator.commandPrint(t, true);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("locals")) {
+                        evaluator.commandLocals();
+                    } else if (cmd.equals("classes")) {
+                        evaluator.commandClasses();
+                    } else if (cmd.equals("class")) {
+                        evaluator.commandClass(t);
+                    } else if (cmd.equals("connectors")) {
+                        evaluator.commandConnectors(Bootstrap.virtualMachineManager());
+                    } else if (cmd.equals("methods")) {
+                        evaluator.commandMethods(t);
+                    } else if (cmd.equals("fields")) {
+                        evaluator.commandFields(t);
+                    } else if (cmd.equals("threads")) {
+                        evaluator.commandThreads(t);
+                    } else if (cmd.equals("thread")) {
+                        evaluator.commandThread(t);
+                    } else if (cmd.equals("suspend")) {
+                        evaluator.commandSuspend(t);
+                    } else if (cmd.equals("resume")) {
+                        evaluator.commandResume(t);
+                    } else if (cmd.equals("cont")) {
+                        evaluator.commandCont();
+                    } else if (cmd.equals("threadgroups")) {
+                        evaluator.commandThreadGroups();
+                    } else if (cmd.equals("threadgroup")) {
+                        evaluator.commandThreadGroup(t);
+                    } else if (cmd.equals("catch")) {
+                        evaluator.commandCatchException(t);
+                    } else if (cmd.equals("ignore")) {
+                        evaluator.commandIgnoreException(t);
+                    } else if (cmd.equals("step")) {
+                        evaluator.commandStep(t);
+                    } else if (cmd.equals("stepi")) {
+                        evaluator.commandStepi();
+                    } else if (cmd.equals("next")) {
+                        evaluator.commandNext();
+                    } else if (cmd.equals("kill")) {
+                        evaluator.commandKill(t);
+                    } else if (cmd.equals("interrupt")) {
+                        evaluator.commandInterrupt(t);
+                    } else if (cmd.equals("trace")) {
+                        evaluator.commandTrace(t);
+                    } else if (cmd.equals("untrace")) {
+                        evaluator.commandUntrace(t);
+                    } else if (cmd.equals("where")) {
+                        evaluator.commandWhere(t, false);
+                    } else if (cmd.equals("wherei")) {
+                        evaluator.commandWhere(t, true);
+                    } else if (cmd.equals("up")) {
+                        evaluator.commandUp(t);
+                    } else if (cmd.equals("down")) {
+                        evaluator.commandDown(t);
+                    } else if (cmd.equals("load")) {
+                        evaluator.commandLoad(t);
+                    } else if (cmd.equals("run")) {
+                        evaluator.commandRun(t);
+                        /*
+                         * Fire up an event handler, if the connection was just
+                         * opened. Since this was done from the run command
+                         * we don't stop the VM on its VM start event (so
+                         * arg 2 is false).
+                         */
+                        if ((handler == null) && Env.connection().isOpen()) {
+                            handler = new EventHandler(this, false);
+                        }
+                    } else if (cmd.equals("memory")) {
+                        evaluator.commandMemory();
+                    } else if (cmd.equals("gc")) {
+                        evaluator.commandGC();
+                    } else if (cmd.equals("stop")) {
+                        evaluator.commandStop(t);
+                    } else if (cmd.equals("clear")) {
+                        evaluator.commandClear(t);
+                    } else if (cmd.equals("watch")) {
+                        evaluator.commandWatch(t);
+                    } else if (cmd.equals("unwatch")) {
+                        evaluator.commandUnwatch(t);
+                    } else if (cmd.equals("list")) {
+                        evaluator.commandList(t);
+                    } else if (cmd.equals("lines")) { // Undocumented command: useful for testing.
+                        evaluator.commandLines(t);
+                    } else if (cmd.equals("classpath")) {
+                        evaluator.commandClasspath(t);
+                    } else if (cmd.equals("use") || cmd.equals("sourcepath")) {
+                        evaluator.commandUse(t);
+                    } else if (cmd.equals("monitor")) {
+                        monitorCommand(t);
+                    } else if (cmd.equals("unmonitor")) {
+                        unmonitorCommand(t);
+                    } else if (cmd.equals("lock")) {
+                        evaluator.commandLock(t);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("threadlocks")) {
+                        evaluator.commandThreadlocks(t);
+                    } else if (cmd.equals("disablegc")) {
+                        evaluator.commandDisableGC(t);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("enablegc")) {
+                        evaluator.commandEnableGC(t);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("save")) { // Undocumented command: useful for testing.
+                        evaluator.commandSave(t);
+                        showPrompt = false;        // asynchronous command
+                    } else if (cmd.equals("bytecodes")) { // Undocumented command: useful for testing.
+                        evaluator.commandBytecodes(t);
+                    } else if (cmd.equals("redefine")) {
+                        evaluator.commandRedefine(t);
+                    } else if (cmd.equals("pop")) {
+                        evaluator.commandPopFrames(t, false);
+                    } else if (cmd.equals("reenter")) {
+                        evaluator.commandPopFrames(t, true);
+                    } else if (cmd.equals("extension")) {
+                        evaluator.commandExtension(t);
+                    } else if (cmd.equals("exclude")) {
+                        evaluator.commandExclude(t);
+                    } else if (cmd.equals("read")) {
+                        readCommand(t);
+                    } else if (cmd.equals("help") || cmd.equals("?")) {
+                        help();
+                    } else if (cmd.equals("version")) {
+                        evaluator.commandVersion(progname,
+                                                 Bootstrap.virtualMachineManager());
+                    } else if (cmd.equals("quit") || cmd.equals("exit")) {
+                        if (handler != null) {
+                            handler.shutdown();
+                        }
+                        Env.shutdown(); 
+                    } else {
+                        MessageOutput.println("Unrecognized command.  Try help...", cmd);
+                    }
+                } catch (VMCannotBeModifiedException rovm) {
+                    MessageOutput.println("Command is not supported on a read-only VM connection", cmd);
+                } catch (UnsupportedOperationException uoe) {
+                    MessageOutput.println("Command is not supported on the target VM", cmd);
+                } catch (VMNotConnectedException vmnse) {
+                    MessageOutput.println("Command not valid until the VM is started with the run command",
+                                          cmd);
+                } catch (Exception e) {
+                    MessageOutput.printException("Internal exception:", e);
+                }
             }
         }
 
@@ -486,7 +594,7 @@ public class TTY implements EventNotifier {
 
         MessageOutput.println("Initializing progname", progname);
        
-        if (Env.connection().isOpen()) {
+        if (Env.connection().isOpen() && Env.vm().canBeModified()) {
             /*
              * Connection opened on startup. Start event handler
              * immediately, telling it (through arg 2) to stop on the 
@@ -583,7 +691,10 @@ public class TTY implements EventNotifier {
         Iterator iter = connectors.iterator();
         while (iter.hasNext()) {
             Connector connector = (Connector)iter.next();
-            if (connector.transport().name().equals("dt_shmem")) {
+            if (connector.transport() == null) {
+                continue;
+            }
+            if ("dt_shmem".equals(connector.transport().name())) {
                 return true;
             }
         }
@@ -746,19 +857,22 @@ public class TTY implements EventNotifier {
                  * specification string based on this decision.
                  */
                 if (supportsSharedMemory()) {
-                    connectSpec = "com.sun.jdi.SharedMemoryListen";
+                    connectSpec = "com.sun.jdi.SharedMemoryListen:";
                     if (address != null) {
-                        connectSpec += (":name=" + address);
+                        connectSpec += ("name=" + address);
                     }
                 } else {
-                    String suboptions = addressToSocketArgs(address);
-                    connectSpec = "com.sun.jdi.SocketListen";
+                    connectSpec = "com.sun.jdi.SocketListen:";
                     if (address != null) {
-                        connectSpec += (":port=" + address);
+                        connectSpec += addressToSocketArgs(address);
                     }
                 }
             } else if (token.equals("-launch")) {
                 launchImmediately = true;
+            } else if (token.equals("-listconnectors")) {
+                Commands evaluator = new Commands();                
+                evaluator.commandConnectors(Bootstrap.virtualMachineManager());
+                return;
             } else if (token.equals("-connect")) {
                 /*
                  * -connect allows the user to pick the connector
@@ -779,7 +893,8 @@ public class TTY implements EventNotifier {
                 usage();
             } else if (token.equals("-version")) {
                 Commands evaluator = new Commands();                
-                evaluator.commandVersion(progname, version);
+                evaluator.commandVersion(progname,
+                                         Bootstrap.virtualMachineManager());
                 System.exit(0);
             } else if (token.startsWith("-")) {
                 usageError("invalid option", token);
