@@ -1,7 +1,7 @@
 /*
- * @(#)TTYResources.java	1.10 03/01/23
+ * @(#)TTYResources.java	1.13 04/02/09
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 
@@ -15,7 +15,7 @@ package com.sun.tools.example.debug.tty;
  * <li> com.sun.tools.example.debug.tty
  * </ol>
  *
- * @version @(#)TTYResources.java	1.10 03/01/23
+ * @version @(#)TTYResources.java	1.13 04/02/09
  */
 public class TTYResources extends java.util.ListResourceBundle {
 
@@ -57,8 +57,16 @@ public class TTYResources extends java.util.ListResourceBundle {
         {"colon mark", ":"},
         {"colon space", ": "},
         {"Command is not supported on the target VM", "Command ''{0}'' is not supported on the target VM"},
+        {"Command is not supported on a read-only VM connection", "Command ''{0}'' is not supported on a read-only VM connection"},
         {"Command not valid until the VM is started with the run command", "Command ''{0}'' is not valid until the VM is started with the ''run'' command"},
         {"Condition must be boolean", "Condition must be boolean"},
+        {"Connector and Transport name", "  Connector: {0}  Transport: {1}"},
+        {"Connector argument nodefault", "    Argument: {0} (no default)"},
+        {"Connector argument default", "    Argument: {0} Default value: {1}"},
+        {"Connector description", "    description: {0}"},
+        {"Connector required argument nodefault", "    Required Argument: {0} (no default)"},
+        {"Connector required argument default", "    Required Argument: {0} Default value: {1}"},
+        {"Connectors available", "Available connectors are:"},
         {"Constant is not a method", "Constant is not a method"},
         {"Could not open:", "Could not open: {0}"},
         {"Current method is native", "Current method is native"},
@@ -147,7 +155,7 @@ public class TTYResources extends java.util.ListResourceBundle {
         {"Method entered:", "Method entered: {0}.{1} "},
         {"Method exited:", "Method exited: {0}.{1} "},
         {"Method is overloaded; specify arguments", "Method {0} is overloaded; specify arguments"},
-        {"minus version", "This is {0} version {1,date,long} {1,time,long}"},
+        {"minus version", "This is {0} version {1,number,integer}.{2,number,integer} (J2SE version {3})"},
         {"Monitor information for thread", "Monitor information for thread {0}:"},
         {"Monitor information for expr", "Monitor information for {0} ({1}):"},
         {"More than one class named", "More than one class named: ''{0}''"},
@@ -258,7 +266,7 @@ public class TTYResources extends java.util.ListResourceBundle {
         {"Unexpected event type", "Unexpected event type"},
         {"unknown", "unknown"},
         {"Unmonitoring", "Unmonitoring {0} "},
-        {"Unrecognized command.  Try help...", "Unrecognized command. Try help..."},
+        {"Unrecognized command.  Try help...", "Unrecognized command: ''{0}''.  Try help..."},
         {"Usage: catch exception", "Usage: catch [uncaught|caught|all] <class id>|<class pattern>"},
         {"Usage: ignore exception", "Usage: ignore [uncaught|caught|all] <class id>|<class pattern>"},
         {"Usage: down [n frames]", "Usage: down [n frames]"},
@@ -277,6 +285,8 @@ public class TTYResources extends java.util.ListResourceBundle {
         {"watch modification of", "watch modification of {0}.{1}"},
         {"zz help text",
              "** command list **\n" +
+             "connectors                -- list available connectors and transports in this VM\n" +
+             "\n" +
              "run [class [args]]        -- start execution of application's main class\n" +
              "\n" +
              "threads [threadgroup]     -- list threads\n" +
@@ -379,6 +389,7 @@ public class TTYResources extends java.util.ListResourceBundle {
              "                      wait for a running VM to connect at any available address using standard connector\n" +
              "    -launch\n" +
              "                      launch VM immediately instead of waiting for ''run'' command\n" +
+             "    -listconnectors   list the connectors available in this VM\n" +
              "    -connect <connector-name>:<name1>=<value1>,...\n" +
              "                      connect to target VM using named connector with listed argument values\n" +
              "    -dbgtrace [flags] print info for debugging {0}\n" +
