@@ -1,7 +1,7 @@
 /*
- * @(#)NumberFormatter.java	1.8 03/01/23
+ * @(#)NumberFormatter.java	1.10 03/12/19
  *
- * Copyright 2003 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2004 Sun Microsystems, Inc. All rights reserved.
  * SUN PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
  */
 package javax.swing.text;
@@ -251,7 +251,7 @@ public class NumberFormatter extends InternationalFormatter {
 
     /**
      * Subclassed to treat the decimal separator, grouping separator,
-     * exponent symbol, percent, permille and currency as literals.
+     * exponent symbol, percent, permille, currency and sign as literals.
      */
     boolean isLiteral(Map attrs) {
         if (!super.isLiteral(attrs)) {
@@ -278,6 +278,9 @@ public class NumberFormatter extends InternationalFormatter {
             if (attrs.get(NumberFormat.Field.CURRENCY) != null) {
                 size--;
             }
+            if (attrs.get(NumberFormat.Field.SIGN) != null) {
+                size--;
+            }
             if (size == 0) {
                 return true;
             }
